import { DatasetProfileDefinitionModel } from '../models/DatasetProfileDefinitionModel';
import { TranslateService } from '@ngx-translate/core';
import { BaseErrorModel } from '../models/error/BaseErrorModel';
import { ActivatedRoute, Router, Params } from '@angular/router';
import { DatasetService } from '../services/dataset/dataset.service';
import { DatasetWizardModel } from '../models/datasets/DatasetWizardModel';
import { DatasetProfileCriteria } from '../models/criteria/dataset-profile/DatasetProfileCriteria';
import { Component, ViewChild, OnInit, AfterViewInit, ViewEncapsulation } from "@angular/core";
import { DataManagementPlanService } from "../services/data-management-plan/data-management-plan.service";
import { FormGroup, Validators, FormBuilder } from "@angular/forms";
import { AutoCompleteConfiguration } from "../shared/components/autocomplete/AutoCompleteConfiguration";
import { DatasetWizardService } from "../services/dataset-wizard/dataset-wizard.service";
import { DataManagementPlanCriteria } from "../models/criteria/data-management-plan/DataManagementPlanCriteria";
import { DataManagementPlanModel } from "../models/data-managemnt-plans/DataManagementPlanModel";
import { JsonSerializer } from "../utilities/JsonSerializer";
import { Observable } from "rxjs/Observable";
import { RequestItem } from "../models/criteria/RequestItem";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { SnackBarNotificationComponent } from "../shared/components/notificaiton/snack-bar-notification.component";
import { DatasetProfileModel } from '../models/datasetprofile/DatasetProfileModel';
import { ExternalSourcesItemModel } from '../models/external-sources/ExternalSourcesItemModel';
import { ExternalSourcesService } from '../services/external-sources/external-sources.service';

@Component({
  selector: 'app-dataset-wizard-component',
  templateUrl: 'dataset-wizard.component.html',
  styleUrls: ['./dataset-wizard.component.scss'],
  providers: [DatasetWizardService, ExternalSourcesService, DataManagementPlanService],
  encapsulation: ViewEncapsulation.None
})
export class DatasetWizardComponent {

  //dmpAutoCompleteConfiguration: AutoCompleteConfiguration;
  //datasetProfileAutoCompleteConfiguration: AutoCompleteConfiguration;
  datasetWizardModel: DatasetWizardModel;
  isNew = true;
  formGroup: FormGroup;

  availableProfiles: DatasetProfileModel[] = [];

  filtereddataRepositoriesAsync: boolean = false;
  filteredRegistriesAsync: boolean = false;
  filteredServicesAsync: boolean = false;
  filteredExternalDatasetsAsync: boolean = false;

  filtereddataRepositories: ExternalSourcesItemModel[];
  filteredRegistries: ExternalSourcesItemModel[];
  filteredServices: ExternalSourcesItemModel[];
  filteredExternalDatasets: ExternalSourcesItemModel[];

  constructor(
    private datasetWizardService: DatasetWizardService,
    private formBuilder: FormBuilder,
    private datasetService: DatasetService,
    private route: ActivatedRoute,
    public snackBar: MatSnackBar,
    public router: Router,
    public language: TranslateService,
    public externalSourcesService: ExternalSourcesService,
    public dataManagementPlanService: DataManagementPlanService
  ) {

  }

  isLinear = false;
  firstStepFormGroup: FormGroup;
  secondFormGroup: FormGroup;

  ngOnInit() {
    let dmpRequestItem: RequestItem<DataManagementPlanCriteria> = new RequestItem();
    dmpRequestItem.criteria = new DataManagementPlanCriteria();


    this.route.params.subscribe((params: Params) => {
      const itemId = params['id'];
      const dmpId = params['dmpId'];
      if (itemId != null) {
        this.isNew = false;
        this.datasetWizardService.getSingle(itemId).map(data => data as DatasetWizardModel)
          .subscribe(data => {
            this.datasetWizardModel = JsonSerializer.fromJSONObject(data, DatasetWizardModel);
            this.formGroup = this.datasetWizardModel.buildForm();
            this.loadDatasetProfiles();
          });
      } else if (dmpId != null) {
        this.isNew = true;
        this.dataManagementPlanService.getSingle(dmpId).map(data => data as DataManagementPlanModel)
          .subscribe(data => {
            this.datasetWizardModel = new DatasetWizardModel();
            setTimeout(() => {
              this.datasetWizardModel.dmp = data;
              this.formGroup = this.datasetWizardModel.buildForm();
              this.loadDatasetProfiles();
            });
          });
      } else {

      }
    });


  }

  loadDatasetProfiles() {
    let datasetProfileRequestItem: RequestItem<DatasetProfileCriteria> = new RequestItem();
    datasetProfileRequestItem.criteria = new DatasetProfileCriteria();
    datasetProfileRequestItem.criteria.id = this.datasetWizardModel.dmp.id;
    this.datasetWizardService.getAvailableProfiles(datasetProfileRequestItem).subscribe(items => {
      this.availableProfiles = JsonSerializer.fromJSONArray(items, DatasetProfileModel);
    });
  }

  public cancel(): void {
    //this.router.navigate(['/dataManagementPlans']);
  }

  getDefinition() {
    if (this.isNew) {
      this.datasetWizardService.getDefinition(this.formGroup.get("profile").get("id").value).subscribe(item => {
        this.datasetWizardModel.datasetProfileDefinition = item;
      })
    }
  }


  formSubmit(): void {
    if (!this.isFormValid()) { return; }
    this.onSubmit();
  }

  public isFormValid() {
    return this.formGroup.valid;
  }

  onSubmit(): void {
    this.datasetWizardService.createDataset(this.formGroup.value).subscribe(
      complete => {
        this.datasetWizardService.getSingle(complete.id).subscribe(
          result => {
            this.datasetWizardModel = JsonSerializer.fromJSONObject(result, DatasetWizardModel);
          }
        )
        this.onCallbackSuccess()
      },
      error => this.onCallbackError(error)
    );
  }

  onCallbackSuccess(): void {
    this.snackBar.openFromComponent(SnackBarNotificationComponent, {
      data: { message: this.isNew ? 'GENERAL.SNACK-BAR.SUCCESSFUL-CREATION' : 'GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE', language: this.language },
      duration: 3000,
      extraClasses: ['snackbar-success']
    })
  }

  onCallbackError(error: any) {
    this.setErrorModel(error.error);
  }

  public setErrorModel(errorModel: BaseErrorModel) {
    Object.keys(errorModel).forEach(item => {
      (<any>this.datasetWizardModel.errorModel)[item] = (<any>errorModel)[item];
    })
  }

  filterdataRepositories(value: string): void {

    this.filtereddataRepositories = undefined;
    if (value) {
      this.filtereddataRepositoriesAsync = true;

      this.externalSourcesService.searchDatasetRepository(value).subscribe(items => {
        this.filtereddataRepositories = items;
        this.filtereddataRepositoriesAsync = false;
      });
    }
  }

  filterRegistries(value: string): void {

    this.filteredRegistries = undefined;
    if (value) {
      this.filteredRegistriesAsync = true;

      this.externalSourcesService.searchDatasetRegistry(value).subscribe(items => {
        this.filteredRegistries = items;
        this.filteredRegistriesAsync = false;

      });
    }
  }

  filterServices(value: string): void {

    this.filteredServices = undefined;
    if (value) {
      this.filteredServicesAsync = true;

      this.externalSourcesService.searchDatasetService(value).subscribe(items => {
        this.filteredServices = items;
        this.filteredServicesAsync = false;
      });
    }
  }


  filterExternalDatasets(value: string): void {

    this.filteredExternalDatasets = undefined;
    if (value) {
      this.filteredExternalDatasetsAsync = true;

      this.externalSourcesService.searchDatasetSExternalDatasetservice(value).subscribe(items => {
        this.filteredExternalDatasets = items;
        this.filteredExternalDatasetsAsync = false;
      });
    }
  }
}